@@ -26,12 +26,8 @@ static struct device_type gip_adapter_type = {
 	.release = gip_adapter_release,
 };
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 3, 0)
-static int gip_client_uevent(struct device *dev, struct kobj_uevent_env *env)
-#else
 static int gip_client_uevent(const struct device *dev,
 			     struct kobj_uevent_env *env)
-#endif
 {
 	struct gip_client *client = to_gip_client(dev);
 	struct gip_classes *classes = client->classes;
@@ -117,24 +113,11 @@ static void gip_bus_remove(struct device *dev)
 	up(&client->drv_lock);
 }
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(5, 13, 0)
-static int gip_bus_remove_compat(struct device *dev)
-{
-	gip_bus_remove(dev);
-
-	return 0;
-}
-#endif
-
 static struct bus_type gip_bus_type = {
 	.name = "xone-gip",
 	.match = gip_bus_match,
 	.probe = gip_bus_probe,
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(5, 13, 0)
-	.remove = gip_bus_remove_compat,
-#else
 	.remove = gip_bus_remove,
-#endif
 };
 
 struct gip_adapter *gip_create_adapter(struct device *parent,
@@ -148,12 +131,7 @@ struct gip_adapter *gip_create_adapter(struct device *parent,
 	if (!adap)
 		return ERR_PTR(-ENOMEM);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 17, 0)
 	adap->id = ida_alloc(&gip_adapter_ida, GFP_KERNEL);
-#else
-	adap->id = ida_simple_get(&gip_adapter_ida, 0, 0, GFP_KERNEL);
-#endif
-
 	if (adap->id < 0) {
 		err = adap->id;
 		goto err_put_device;
@@ -185,11 +163,7 @@ err_destroy_queue:
 	destroy_workqueue(adap->clients_wq);
 
 err_remove_ida:
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 17, 0)
 	ida_free(&gip_adapter_ida, adap->id);
-#else
-	ida_simple_remove(&gip_adapter_ida, adap->id);
-#endif
 
 err_put_device:
 	put_device(&adap->dev);
@@ -226,11 +200,7 @@ void gip_destroy_adapter(struct gip_adapter *adap)
 		device_unregister(&client->dev);
 	}
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 17, 0)
 	ida_free(&gip_adapter_ida, adap->id);
-#else
-	ida_simple_remove(&gip_adapter_ida, adap->id);
-#endif
 	destroy_workqueue(adap->clients_wq);
 
 	dev_dbg(&adap->dev, "%s: unregistered\n", __func__);
@@ -254,7 +224,7 @@ static void gip_register_client(struct work_struct *work)
 	 * The unfortunate workaround, that at least works reliably is to add a
 	 * delay here. Since this is for human input device, one second is fine.
 	 */
-	msleep(1000);
+	msleep(700);
 
 	client->dev.parent = &client->adapter->dev;
 	client->dev.type = &gip_client_type;
@@ -379,5 +349,5 @@ module_exit(gip_bus_exit);
 
 MODULE_AUTHOR("Severin von Wnuck-Lipinski <severinvonw@outlook.de>");
 MODULE_DESCRIPTION("xone GIP driver");
-MODULE_VERSION("0.4.11");
+MODULE_VERSION("0.5.0");
 MODULE_LICENSE("GPL");
