@@ -70,6 +70,26 @@ enum gip_audio_volume_mute {
 	GIP_AUD_VOLUME_MIC_MUTED = 0x05,
 };
 
+static uint gip_audio_sample_rate[] = {
+	0,
+	8000,
+	8000,
+	12000,
+	12000,
+	16000,
+	16000,
+	20000,
+	20000,
+	24000,
+	24000,
+	32000,
+	32000,
+	40000,
+	40000,
+	48000,
+	48000,
+};
+
 enum gip_extended_command {
 	GIP_EXT_GET_CAPABILITIES = 0x00,
 	GIP_EXT_GET_TELEMETRY = 0x01,
@@ -182,7 +202,7 @@ struct gip_pkt_serial_number {
 } __packed;
 
 struct gip_pkt_audio_samples {
-	__le16 length_out;
+	__le16 flow_rate;
 	u8 samples[];
 } __packed;
 
@@ -482,7 +502,7 @@ int gip_send_authenticate(struct gip_client *client, void *pkt, u32 len,
 }
 
 static int gip_set_audio_format_chat(struct gip_client *client,
-				     enum gip_audio_format_chat in_out)
+				     enum gip_audio_format in_out)
 {
 	struct gip_header hdr = {};
 	struct gip_pkt_audio_format_chat pkt = {};
@@ -525,7 +545,7 @@ int gip_suggest_audio_format(struct gip_client *client,
 	/* special handling for the chat headset */
 	if (chat)
 		err = gip_set_audio_format_chat(client,
-						GIP_AUD_FORMAT_CHAT_24KHZ);
+						GIP_AUD_FORMAT_12KHZ_STEREO);
 	else
 		err = gip_set_audio_format(client, in, out);
 
@@ -624,10 +644,10 @@ static void gip_copy_audio_samples(struct gip_client *client,
 		dest = buf + i * cfg->packet_size;
 
 		/* sequence number is always greater than zero */
-		do {
-			hdr.sequence = client->adapter->audio_sequence++;
-		} while (!hdr.sequence);
+		if (!++client->adapter->audio_sequence)
+			++client->adapter->audio_sequence;
 
+		hdr.sequence = client->adapter->audio_sequence;
 		gip_encode_header(&hdr, dest);
 		memcpy(dest + hdr_len, src, cfg->fragment_size);
 	}
@@ -785,25 +805,16 @@ static int gip_make_audio_config(struct gip_client *client,
 {
 	struct gip_header hdr = {};
 
-	switch (cfg->format) {
-	case GIP_AUD_FORMAT_16KHZ_MONO:
-		cfg->channels = 1;
-		cfg->sample_rate = 16000;
-		break;
-	case GIP_AUD_FORMAT_24KHZ_MONO:
-		cfg->channels = 1;
-		cfg->sample_rate = 24000;
-		break;
-	case GIP_AUD_FORMAT_48KHZ_STEREO:
-		cfg->channels = 2;
-		cfg->sample_rate = 48000;
-		break;
-	default:
-		gip_err(client, "%s: unknown format: 0x%02x\n",
-			__func__, cfg->format);
+	if (cfg->format <= 0 || cfg->format > GIP_AUD_FORMAT_48KHZ_STEREO) {
+		gip_err(client, "%s: unknown format: 0x%02x\n", __func__,
+			cfg->format);
 		return -ENOTSUPP;
 	}
 
+	/* even-indexed formats are stereo, uneven are mono */
+	cfg->channels = 2 - (cfg->format & 1);
+	cfg->sample_rate = gip_audio_sample_rate[cfg->format];
+
 	cfg->buffer_size = cfg->sample_rate * cfg->channels *
 			   sizeof(s16) * GIP_AUDIO_INTERVAL / MSEC_PER_SEC;
 	cfg->fragment_size = cfg->buffer_size /
@@ -813,6 +824,9 @@ static int gip_make_audio_config(struct gip_client *client,
 	hdr.packet_length = cfg->fragment_size;
 	cfg->packet_size = gip_get_header_length(&hdr) + cfg->fragment_size;
 
+	/* set initial flow rate value */
+	cfg->flow_rate = cfg->buffer_size;
+
 	gip_dbg(client, "%s: rate=%d/%d, buffer=%d\n", __func__,
 		cfg->sample_rate, cfg->channels, cfg->buffer_size);
 
@@ -842,7 +856,7 @@ static struct gip_info_element *gip_parse_info_element(u8 *data, u32 len,
 	if (len < off + total)
 		return ERR_PTR(-EINVAL);
 
-	elem = kzalloc(sizeof(*elem) + total, GFP_ATOMIC);
+	elem = kzalloc(struct_size(elem, data, total), GFP_ATOMIC);
 	if (!elem)
 		return ERR_PTR(-ENOMEM);
 
@@ -983,8 +997,7 @@ static int gip_parse_classes(struct gip_client *client,
 	if (!count)
 		return -EINVAL;
 
-	classes = kzalloc(sizeof(*classes) + sizeof(char *) * count,
-			  GFP_ATOMIC);
+	classes = kzalloc(struct_size(classes, strings, count), GFP_ATOMIC);
 	if (!classes)
 		return -ENOMEM;
 
@@ -1342,7 +1355,7 @@ static int gip_handle_pkt_audio_format_chat(struct gip_client *client,
 		return -EINVAL;
 
 	/* chat headsets apparently default to 24 kHz */
-	if (pkt->in_out != GIP_AUD_FORMAT_CHAT_24KHZ ||
+	if (pkt->in_out != GIP_AUD_FORMAT_12KHZ_STEREO ||
 	    in->buffer_size || out->buffer_size)
 		return -EPROTO;
 
@@ -1508,6 +1521,7 @@ static int gip_handle_pkt_audio_samples(struct gip_client *client,
 					void *data, u32 len)
 {
 	struct gip_pkt_audio_samples *pkt = data;
+	struct gip_audio_config *out = &client->audio_config_out;
 	int err = 0;
 
 	if (len < sizeof(*pkt))
@@ -1516,6 +1530,10 @@ static int gip_handle_pkt_audio_samples(struct gip_client *client,
 	if (down_trylock(&client->drv_lock))
 		return -EBUSY;
 
+	out->flow_rate = le16_to_cpu(pkt->flow_rate);
+	if (out->flow_rate != out->buffer_size)
+		gip_dbg(client, "%s: Unusual flow rate control -> %u", __func__, out->flow_rate);
+
 	if (client->drv && client->drv->ops.audio_samples)
 		err = client->drv->ops.audio_samples(client, pkt->samples,
 						     len - sizeof(*pkt));
