@@ -5,7 +5,6 @@
 
 #include <linux/module.h>
 #include <linux/hrtimer.h>
-#include <linux/vmalloc.h>
 #include <linux/version.h>
 #include <sound/core.h>
 #include <sound/initval.h>
@@ -68,8 +67,6 @@ struct gip_headset {
 		snd_pcm_uframes_t pointer;
 		snd_pcm_uframes_t period;
 	} playback, capture;
-
-	struct snd_card *card;
 };
 
 static int gip_headset_pcm_open(struct snd_pcm_substream *sub)
@@ -89,7 +86,9 @@ static int gip_headset_pcm_open(struct snd_pcm_substream *sub)
 	hw.channels_max = cfg->channels;
 	hw.buffer_bytes_max = cfg->buffer_size * GIP_HS_NUM_BUFFERS;
 	hw.period_bytes_min = cfg->buffer_size;
-	hw.period_bytes_max = cfg->buffer_size;
+	hw.period_bytes_max = cfg->buffer_size * 2;
+	hw.periods_min = 1;
+	hw.periods_max = 8;
 
 	sub->runtime->hw = hw;
 
@@ -101,39 +100,6 @@ static int gip_headset_pcm_close(struct snd_pcm_substream *sub)
 	return 0;
 }
 
-static int gip_headset_pcm_hw_params(struct snd_pcm_substream *sub,
-				     struct snd_pcm_hw_params *params)
-{
-	struct snd_pcm_runtime *runtime = sub->runtime;
-	size_t size = params_buffer_bytes(params);
-
-	if (runtime->dma_area) {
-		if (runtime->dma_bytes >= size)
-			return 0; /* Already large enough */
-		vfree(runtime->dma_area);
-	}
-	runtime->dma_area = vzalloc(size);
-	if (!runtime->dma_area)
-		return -ENOMEM;
-	runtime->dma_bytes = size;
-	return 1;
-}
-
-static int gip_headset_pcm_hw_free(struct snd_pcm_substream *sub)
-{
-	struct snd_pcm_runtime *runtime = sub->runtime;
-
-	vfree(runtime->dma_area);
-	runtime->dma_area = NULL;
-	return 0;
-}
-
-static struct page *gip_headset_pcm_get_page(struct snd_pcm_substream *sub,
-					     unsigned long offset)
-{
-	return vmalloc_to_page(sub->runtime->dma_area + offset);
-}
-
 static int gip_headset_pcm_prepare(struct snd_pcm_substream *sub)
 {
 	return 0;
@@ -186,13 +152,9 @@ static snd_pcm_uframes_t gip_headset_pcm_pointer(struct snd_pcm_substream *sub)
 static const struct snd_pcm_ops gip_headset_pcm_ops = {
 	.open = gip_headset_pcm_open,
 	.close = gip_headset_pcm_close,
-	.ioctl = snd_pcm_lib_ioctl,
-	.hw_params = gip_headset_pcm_hw_params,
-	.hw_free = gip_headset_pcm_hw_free,
 	.prepare = gip_headset_pcm_prepare,
 	.trigger = gip_headset_pcm_trigger,
 	.pointer = gip_headset_pcm_pointer,
-	.page = gip_headset_pcm_get_page,
 };
 
 static bool gip_headset_advance_pointer(struct gip_headset_stream *stream,
@@ -334,8 +296,8 @@ static int gip_headset_init_pcm(struct gip_headset *headset)
 	struct snd_pcm *pcm;
 	int err;
 
-	err = snd_card_new(&headset->client->dev, SNDRV_DEFAULT_IDX1,
-			   SNDRV_DEFAULT_STR1, THIS_MODULE, 0, &card);
+	err = snd_devm_card_new(&headset->client->dev, SNDRV_DEFAULT_IDX1,
+				SNDRV_DEFAULT_STR1, THIS_MODULE, 0, &card);
 	if (err)
 		return err;
 
@@ -344,8 +306,6 @@ static int gip_headset_init_pcm(struct gip_headset *headset)
 	snprintf(card->longname, sizeof(card->longname), "%s at %s",
 		 GIP_HS_NAME, dev_name(&headset->client->dev));
 
-	headset->card = card;
-
 	err = snd_pcm_new(card, GIP_HS_NAME, 0, 1, 1, &pcm);
 	if (err)
 		return err;
@@ -355,6 +315,7 @@ static int gip_headset_init_pcm(struct gip_headset *headset)
 
 	snd_pcm_set_ops(pcm, SNDRV_PCM_STREAM_PLAYBACK, &gip_headset_pcm_ops);
 	snd_pcm_set_ops(pcm, SNDRV_PCM_STREAM_CAPTURE, &gip_headset_pcm_ops);
+	snd_pcm_set_managed_buffer_all(pcm, SNDRV_DMA_TYPE_VMALLOC, NULL, 0, 0);
 
 	return snd_card_register(card);
 }
@@ -440,7 +401,7 @@ static void gip_headset_register(struct work_struct *work)
 		return;
 
 	dev_dbg(&client->dev, "%s: init pcm device.\n", __func__);
-	err = gip_headset_init_pcm(headset);
+	err = snd_card_free_on_error(&client->dev, gip_headset_init_pcm(headset));
 	if (err) {
 		dev_err(&client->dev, "%s: init PCM failed: %d\n",
 			__func__, err);
@@ -466,6 +427,14 @@ static void gip_headset_register(struct work_struct *work)
 		return;
 	}
 
+	dev_dbg(&client->dev, "%s: init audio in.\n", __func__);
+	err = gip_init_audio_in(client);
+	if (err) {
+		dev_err(&client->dev, "%s: init audio in failed: %d\n",
+			__func__, err);
+		return;
+	}
+
 	/* start audio timer */
 	hrtimer_start(&headset->timer, 0, HRTIMER_MODE_REL);
 }
@@ -574,28 +543,22 @@ static int gip_headset_probe(struct gip_client *client)
 	INIT_DELAYED_WORK(&headset->work_power_on, gip_headset_power_on);
 	INIT_WORK(&headset->work_register, gip_headset_register);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(6,15,0)
-	hrtimer_setup(&headset->timer, gip_headset_send_samples,
-		      CLOCK_MONOTONIC, HRTIMER_MODE_REL);
-	hrtimer_setup(&headset->start_audio_timer, gip_headset_start_audio,
-		      CLOCK_MONOTONIC, HRTIMER_MODE_REL);
-#else
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6,15,0)
 	hrtimer_init(&headset->timer, CLOCK_MONOTONIC, HRTIMER_MODE_REL);
 	headset->timer.function = gip_headset_send_samples;
 	hrtimer_init(&headset->start_audio_timer, CLOCK_MONOTONIC, HRTIMER_MODE_REL);
 	headset->start_audio_timer.function = gip_headset_start_audio;
+#else
+	hrtimer_setup(&headset->timer, gip_headset_send_samples,
+		      CLOCK_MONOTONIC, HRTIMER_MODE_REL);
+	hrtimer_setup(&headset->start_audio_timer, gip_headset_start_audio,
+		      CLOCK_MONOTONIC, HRTIMER_MODE_REL);
 #endif
 
 	err = gip_enable_audio(client);
 	if (err)
 		return err;
 
-	err = gip_init_audio_in(client);
-	if (err) {
-		gip_disable_audio(client);
-		return err;
-	}
-
 	dev_set_drvdata(&client->dev, headset);
 
 	/* start audio configuration */
@@ -614,11 +577,6 @@ static void gip_headset_remove(struct gip_client *client)
 	hrtimer_cancel(&headset->timer);
 	hrtimer_cancel(&headset->start_audio_timer);
 	gip_disable_audio(client);
-
-	if (headset->card) {
-		snd_card_disconnect(headset->card);
-		snd_card_free_when_closed(headset->card);
-	}
 }
 
 static struct gip_driver gip_headset_driver = {
@@ -640,5 +598,5 @@ module_gip_driver(gip_headset_driver);
 MODULE_ALIAS("gip:Windows.Xbox.Input.Headset");
 MODULE_AUTHOR("Severin von Wnuck-Lipinski <severinvonw@outlook.de>");
 MODULE_DESCRIPTION("xone GIP headset driver");
-MODULE_VERSION("0.4.11");
+MODULE_VERSION("0.5.0");
 MODULE_LICENSE("GPL");
