@@ -26,16 +26,12 @@
 
 #define XONE_DONGLE_MAX_CLIENTS 16
 
-#define XONE_DONGLE_PAIRING_TIMEOUT msecs_to_jiffies(30000)
+#define XONE_DONGLE_PAIRING_TIMEOUT msecs_to_jiffies(60000)
 #define XONE_DONGLE_PWR_OFF_TIMEOUT msecs_to_jiffies(5000)
 #define XONE_DONGLE_FW_REQ_TIMEOUT_MS 3000
 #define XONE_DONGLE_FW_REQ_RETRIES 11 // 30 seconds
 #define XONE_DONGLE_FW_LOAD_RETRIES 3
 
-#define XONE_DONGLE_OFFICIAL_VENDOR 0x045e
-#define XONE_DONGLE_OFFICIAL_PRODUCT 0x02fe
-#define XONE_DONGLE_KNOCKOFF_PRODUCT1 0x02e6
-
 enum xone_dongle_queue {
 	XONE_DONGLE_QUEUE_DATA = 0x00,
 	XONE_DONGLE_QUEUE_AUDIO = 0x02,
@@ -62,14 +58,16 @@ struct xone_dongle_client {
 	struct gip_adapter *adapter;
 };
 
+enum xone_dongle_event_type {
+	XONE_DONGLE_EVT_ADD_CLIENT,
+	XONE_DONGLE_EVT_REMOVE_CLIENT,
+	XONE_DONGLE_EVT_PAIR_CLIENT,
+	XONE_DONGLE_EVT_ENABLE_PAIRING,
+	XONE_DONGLE_EVT_ENABLE_ENCRYPTION,
+};
+
 struct xone_dongle_event {
-	enum xone_dongle_event_type {
-		XONE_DONGLE_EVT_ADD_CLIENT,
-		XONE_DONGLE_EVT_REMOVE_CLIENT,
-		XONE_DONGLE_EVT_PAIR_CLIENT,
-		XONE_DONGLE_EVT_ENABLE_PAIRING,
-		XONE_DONGLE_EVT_ENABLE_ENCRYPTION,
-	} type;
+	enum xone_dongle_event_type type;
 
 	struct xone_dongle *dongle;
 	u8 address[ETH_ALEN];
@@ -230,7 +228,6 @@ static struct gip_adapter_ops xone_dongle_adapter_ops = {
 
 static int xone_dongle_toggle_pairing(struct xone_dongle *dongle, bool enable)
 {
-	struct usb_interface *intf = to_usb_interface(dongle->mt.dev);
 	enum xone_mt76_led_mode led;
 	int err = 0;
 
@@ -255,11 +252,6 @@ static int xone_dongle_toggle_pairing(struct xone_dongle *dongle, bool enable)
 	if (err)
 		goto err_unlock;
 
-	if (enable)
-		usb_autopm_get_interface(intf);
-	else
-		usb_autopm_put_interface(intf);
-
 	dev_dbg(dongle->mt.dev, "%s: enabled=%d\n", __func__, enable);
 	dongle->pairing = enable;
 
@@ -296,7 +288,7 @@ static ssize_t xone_dongle_pairing_show(struct device *dev,
 	struct usb_interface *intf = to_usb_interface(dev);
 	struct xone_dongle *dongle = usb_get_intfdata(intf);
 
-	return sprintf(buf, "%d\n", dongle->pairing);
+	return sysfs_emit(buf, "%d\n", dongle->pairing);
 }
 
 static ssize_t xone_dongle_pairing_store(struct device *dev,
@@ -778,8 +770,8 @@ static int xone_dongle_process_buffer(struct xone_dongle *dongle,
 	if (err) {
 		dev_err(dongle->mt.dev, "%s: process failed: %d\n",
 			__func__, err);
-		print_hex_dump_debug("xone-dongle packet: ", DUMP_PREFIX_NONE,
-				     16, 1, data, len, false);
+		print_hex_dump_bytes("xone-dongle packet: ", DUMP_PREFIX_NONE,
+				     data, len);
 	}
 
 	dev_kfree_skb(skb);
@@ -918,20 +910,10 @@ static void xone_dongle_fw_load(struct work_struct *work)
 
 	struct xone_mt76 *mt = &dongle->mt;
 	const struct firmware *fw;
-	char fwname[25];
+	char fwname[21];
 	int err;
 
-	switch (dongle->product) {
-	case XONE_DONGLE_KNOCKOFF_PRODUCT1:
-		snprintf(fwname, 25, "xow_dongle_%04x_%04x.bin", dongle->vendor,
-			 dongle->product);
-		break;
-
-	case XONE_DONGLE_OFFICIAL_PRODUCT:
-	default:
-		snprintf(fwname, 15, "xow_dongle.bin");
-	}
-
+	sprintf(fwname, "xone_dongle_%04x.bin", dongle->product);
 	err = xone_dongle_fw_requester(&fw, dongle, fwname);
 	if (dongle->fw_state == XONE_DONGLE_FW_STATE_STOP_LOADING) {
 		dongle->fw_state = XONE_DONGLE_FW_STATE_ERROR;
@@ -1310,5 +1292,5 @@ module_usb_driver(xone_dongle_driver);
 MODULE_DEVICE_TABLE(usb, xone_dongle_id_table);
 MODULE_AUTHOR("Severin von Wnuck-Lipinski <severinvonw@outlook.de>");
 MODULE_DESCRIPTION("xone dongle driver");
-MODULE_VERSION("0.4.11");
+MODULE_VERSION("0.5.0");
 MODULE_LICENSE("GPL");
